indexing

   description: "Abstract file system that should work on POSIX and Windows platforms."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


deferred class

   ABSTRACT_FILE_SYSTEM


inherit

   STDC_FILE_SYSTEM
      redefine
         is_modifiable,
         is_readable
      end


feature -- directory access

   change_directory, chdir (a_directory: STRING) is
         -- Changes the current working directory
      do
         set_portable_path (a_directory)
         safe_call (abstract_chdir (sh.string_to_pointer (portable_path)))
      end

   current_directory, getcwd, pwd: STRING is
         -- The current directory
      local
         buf: STDC_BUFFER
         r: POINTER
         buf_size: INTEGER
      do
         buf_size := 256
         create buf.allocate (buf_size)
         r := abstract_getcwd (buf.ptr, buf_size)
         if r = default_pointer then
            raise_posix_error
         end
         Result := sh.pointer_to_string (r)
      end

   make_directory, mkdir (a_directory: STRING) is
         -- Makes a directory, only accessible by owner
      do
         set_portable_path (a_directory)
         safe_call (abstract_mkdir (sh.string_to_pointer (portable_path)))
      end

   remove_directory, rmdir (a_directory: STRING) is
         -- Removes an empty directory, see also `force_remove_directory'
      require
         valid_directory_name: a_directory /= Void and then not a_directory.is_empty
         directory_is_empty: True
      do
         set_portable_path (a_directory)
         safe_call (abstract_rmdir (sh.string_to_pointer (portable_path)))
      end

   force_remove_directory (a_directory: STRING) is
         -- Removes a directory, even when not empty
      require
         valid_path: a_directory /= Void and then not a_directory.is_empty
      local
         dir: ABSTRACT_DIRECTORY
         entry: STRING
         full_child_path: STRING
         is_special_file: BOOLEAN
      do
         if is_existing (a_directory) then
            full_child_path := sh.make_with_capacity (256)
            dir := browse_directory (a_directory)
            from
               dir.start
            until
               dir.exhausted
            loop
               entry := dir.item
               is_special_file :=
                  entry.is_equal (".")
                  or else entry.is_equal ("..")
               if not is_special_file then
                  sh.clear (full_child_path)
                  full_child_path.append_string (a_directory)
                  full_child_path.append_character (path_separator)
                  full_child_path.append_string (entry)
                  if status (full_child_path).is_directory then
                     force_remove_directory (full_child_path)
                  else
                     remove_file (full_child_path)
                  end
               end
               dir.forth
            end
            dir.close
            -- directory is empty now
            remove_directory (a_directory)
         end
      end


feature -- file statistics

   status (a_path: STRING): ABSTRACT_STATUS is
         -- Gets information about a file
      require
         valid_path: a_path /= Void and then not a_path.is_empty
         existing_file: is_existing (a_path)
      deferred
      ensure
         status_returned: Result /= Void
      end


feature -- directory browsing

   browse_directory (a_path: STRING): ABSTRACT_DIRECTORY is
         -- Gets information about a directory
      require
         valid_path: a_path /= Void and then not a_path.is_empty
         path_is_directory: security.error_handling.exceptions_enabled and then
                            (status (a_path)).is_directory
      deferred
      ensure
         directory_returned: Result /= Void
      end


feature -- accessibility of files

   last_access_result: INTEGER
         -- value of last access test

   is_accessible, access (a_path: STRING; a_mode: INTEGER): BOOLEAN is
         -- Tests for file accessibility
      do
         set_portable_path (a_path)
         last_access_result := abstract_access (sh.string_to_pointer (portable_path), a_mode)
         Result := last_access_result = 0
      end

   is_directory (a_path: STRING): BOOLEAN is
         -- return True if `a_path' exists and if it is a directory
      do
         Result :=
            is_existing (a_path) and then
            status (a_path).is_directory
      end

   is_existing (a_path: STRING): BOOLEAN is
         -- tests if file does exist, not if it is readable or writable by
         -- this program!
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, abstract_F_OK)
      end

   is_empty (a_path: STRING): BOOLEAN is
         -- True if file exists and has a size equal to zero.
      require
         exists: is_existing (a_path)
      do
         Result := status (a_path).size = 0
      end

   is_executable (a_path: STRING): BOOLEAN is
         -- tests if file is executable by this program
      do
         Result := is_accessible (a_path, abstract_X_OK)
      end

   is_modifiable (a_path: STRING): BOOLEAN is
         -- tests if file is readable and writable by this program
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, abstract_R_OK + abstract_W_OK)
      end

   is_readable (a_path: STRING): BOOLEAN is
         -- tests if file is readable by this program
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, abstract_R_OK)
      end

   is_writable (a_path: STRING): BOOLEAN is
         -- tests if file is writable by this program
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, abstract_W_OK)
      end


feature -- various

   is_case_sensitive: BOOLEAN is
         -- is file system case sensitive or not?
         -- This query is dedicated to jwz
      deferred
      end

   path_separator: CHARACTER is
         -- What is the path separator?
      deferred
      end


feature -- file system properties

   temporary_directory: STRING is
         -- returns temporary directory.
      deferred
      ensure
         directory_returned: Result /= Void
         directory_exists: is_directory (Result)
         directory_is_writable: is_modifiable (Result)
         last_char_not_separator: Result.item (Result.count) /= path_separator
      end


feature {NONE} -- abstract C interface

   abstract_access (a_path: POINTER; a_mode: INTEGER): INTEGER is
         -- Tests for file accessibility
      require
         valid_path: a_path /= default_pointer
      deferred
      end

   abstract_chdir (a_path: POINTER): INTEGER is
         -- Changes the current working directory
      require
         valid_path: a_path /= default_pointer
      deferred
      end

   abstract_getcwd (buf: POINTER; size: INTEGER): POINTER is
         -- Gets current working directory
      deferred
      end

   abstract_mkdir (a_path: POINTER): INTEGER is
         -- Makes a directory
      require
         valid_path: a_path /= default_pointer
      deferred
      end

   abstract_rmdir (a_path: POINTER): INTEGER is
         -- Removes a directory
      require
         valid_path: a_path /= default_pointer
      deferred
      end

   abstract_F_OK: INTEGER is
         -- existence
      deferred
      end

   abstract_R_OK: INTEGER is
         -- read permission
      deferred
      end

   abstract_W_OK: INTEGER is
         -- write permission
      deferred
      end

   abstract_X_OK: INTEGER is
         -- execute permission
      deferred
      end

end -- class ABSTRACT_FILE_SYSTEM
